@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: easy.c,v 1.24 2001/09/17 14:10:38 bagder Exp $
+ * $Id: easy.c,v 1.26 2001/10/19 11:57:50 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -293,6 +293,12 @@ CURL *curl_easy_duphandle(CURL *incurl)
   outcurl->progress.flags    = data->progress.flags;
   outcurl->progress.callback = data->progress.callback;
 
+  if(data->cookies)
+    /* If cookies are enabled in the parent handle, we enable them
+       in the clone as well! */
+    outcurl->cookies = Curl_cookie_init(data->cookies->filename,
+                                        outcurl->cookies);
+
   /* duplicate all values in 'change' */
   if(data->change.url) {
     outcurl->change.url = strdup(data->change.url);
@@ -314,6 +320,6 @@ CURL *curl_easy_duphandle(CURL *incurl)
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
