@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: getinfo.c,v 1.12 2001/09/07 04:01:32 bumblebury Exp $
+ * $Id: getinfo.c,v 1.15 2001/11/20 15:00:50 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -43,10 +43,11 @@ CURLcode Curl_initinfo(struct SessionHandle *data)
   pro->t_nslookup = 0;
   pro->t_connect = 0;
   pro->t_pretransfer = 0;
+  pro->t_starttransfer = 0;
 
   info->httpcode = 0;
   info->httpversion=0;
-  info->filetime=0;
+  info->filetime=-1; /* -1 is an illegal time and thus means unknown */
 
   return CURLE_OK;
 }
@@ -107,6 +108,9 @@ CURLcode Curl_getinfo(struct SessionHandle *data, CURLINFO info, ...)
   case CURLINFO_PRETRANSFER_TIME:
     *param_doublep =  data->progress.t_pretransfer;
     break;
+  case CURLINFO_STARTTRANSFER_TIME:
+    *param_doublep = data->progress.t_starttransfer;
+    break;
   case CURLINFO_SIZE_UPLOAD:
     *param_doublep =  data->progress.uploaded;
     break;
@@ -138,6 +142,6 @@ CURLcode Curl_getinfo(struct SessionHandle *data, CURLINFO info, ...)
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
