@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: http.c,v 1.72 2001/09/18 15:29:57 bagder Exp $
+ * $Id: http.c,v 1.80 2001/11/16 11:21:50 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -125,22 +125,27 @@ send_buffer *add_buffer_init(void)
  * add_buffer_send() sends a buffer and frees all associated memory.
  */
 static
-size_t add_buffer_send(int sockfd, struct connectdata *conn, send_buffer *in)
+CURLcode add_buffer_send(int sockfd, struct connectdata *conn, send_buffer *in,
+                         long *bytes_written)
 {
   size_t amount;
+  CURLcode result;
+
   if(conn->data->set.verbose) {
     fputs("> ", conn->data->set.err);
     /* this data _may_ contain binary stuff */
     fwrite(in->buffer, in->size_used, 1, conn->data->set.err);
   }
 
-  Curl_write(conn, sockfd, in->buffer, in->size_used, &amount);
+  result = Curl_write(conn, sockfd, in->buffer, in->size_used, &amount);
 
   if(in->buffer)
     free(in->buffer);
   free(in);
 
-  return amount;
+  *bytes_written = amount;
+
+  return result;
 }
 
 
@@ -199,28 +204,6 @@ CURLcode add_buffer(send_buffer *in, const void *inptr, size_t size)
 /* end of the add_buffer functions */
 /* ------------------------------------------------------------------------- */
 
-/*
- * Read everything until a newline.
- */
-
-static
-int GetLine(int sockfd, char *ptr, struct connectdata *conn)
-{
-  ssize_t nread;
-
-  /* get us a full line, terminated with a newline */
-  for(nread=0; (nread<BUFSIZE); nread++, ptr++) {
-    if((CURLE_OK != Curl_read(conn, sockfd, ptr, 1, &nread)) ||
-       (nread <= 0) || (*ptr == '\n'))
-      break;
-  }
-  *ptr=0; /* zero terminate */
-  
-  return nread>0?nread:0;
-}
-
-
-
 /*
  * This function checks the linked list of custom HTTP headers for a particular
  * header (prefix).
@@ -251,33 +234,141 @@ CURLcode Curl_ConnectHTTPProxyTunnel(struct connectdata *conn,
   int httperror=0;
   int subversion=0;
   struct SessionHandle *data=conn->data;
+  CURLcode result;
+
+  int nread;   /* total size read */
+  int perline; /* count bytes per line */
+  bool keepon=TRUE;
+  ssize_t gotbytes;
+  char *ptr;
+  int timeout = 3600; /* default timeout in seconds */
+  struct timeval interval;
+  fd_set rkeepfd;
+  fd_set readfd;
+  char *line_start;
+
+#define SELECT_OK      0
+#define SELECT_ERROR   1
+#define SELECT_TIMEOUT 2
+  int error = SELECT_OK;
 
   infof(data, "Establish HTTP proxy tunnel to %s:%d\n", hostname, remote_port);
 
   /* OK, now send the connect request to the proxy */
-  Curl_sendf(tunnelsocket, conn,
-             "CONNECT %s:%d HTTP/1.0\015\012"
-             "%s"
-             "%s"
-             "\r\n",
-             hostname, remote_port,
-             (conn->bits.proxy_user_passwd)?conn->allocptr.proxyuserpwd:"",
-             (data->set.useragent?conn->allocptr.uagent:"")
-             );
+  result =
+    Curl_sendf(tunnelsocket, conn,
+               "CONNECT %s:%d HTTP/1.0\015\012"
+               "%s"
+               "%s"
+               "\r\n",
+               hostname, remote_port,
+               (conn->bits.proxy_user_passwd)?conn->allocptr.proxyuserpwd:"",
+               (data->set.useragent?conn->allocptr.uagent:"")
+               );
+  if(result) {
+    failf(data, "Failed sending CONNECT to proxy");
+    return result;
+  }
 
-  /* wait for the proxy to send us a HTTP/1.0 200 OK header */
-  while(GetLine(tunnelsocket, data->state.buffer, conn)) {
-    if('\r' == data->state.buffer[0])
-      break; /* end of headers */
-    if(data->set.verbose)
-      fprintf(data->set.err, "< %s\n", data->state.buffer);
+  /* Now, read the full reply we get from the proxy */
 
-    if(2 == sscanf(data->state.buffer, "HTTP/1.%d %d",
-                   &subversion,
-                   &httperror)) {
-      ;
+
+  if(data->set.timeout) {
+    /* if timeout is requested, find out how much remaining time we have */
+    timeout = data->set.timeout - /* timeout time */
+      Curl_tvdiff(Curl_tvnow(), conn->now)/1000; /* spent time */
+    if(timeout <=0 ) {
+      failf(data, "Transfer aborted due to timeout");
+      return -SELECT_TIMEOUT; /* already too little time */
     }
   }
+
+  FD_ZERO (&readfd);		/* clear it */
+  FD_SET (tunnelsocket, &readfd);     /* read socket */
+
+  /* get this in a backup variable to be able to restore it on each lap in the
+     select() loop */
+  rkeepfd = readfd;
+
+  ptr=data->state.buffer;
+  line_start = ptr;
+
+  nread=0;
+  perline=0;
+  keepon=TRUE;
+
+  while((nread<BUFSIZE) && (keepon && !error)) {
+    readfd = rkeepfd;		   /* set every lap */
+    interval.tv_sec = timeout;
+    interval.tv_usec = 0;
+
+    switch (select (tunnelsocket+1, &readfd, NULL, NULL, &interval)) {
+    case -1: /* select() error, stop reading */
+      error = SELECT_ERROR;
+      failf(data, "Transfer aborted due to select() error");
+      break;
+    case 0: /* timeout */
+      error = SELECT_TIMEOUT;
+      failf(data, "Transfer aborted due to timeout");
+      break;
+    default:
+      /*
+       * This code previously didn't use the kerberos sec_read() code
+       * to read, but when we use Curl_read() it may do so. Do confirm
+       * that this is still ok and then remove this comment!
+       */
+      if(CURLE_OK != Curl_read(conn, tunnelsocket, ptr, BUFSIZE-nread,
+                               &gotbytes))
+        keepon = FALSE;
+      else if(gotbytes <= 0) {
+        keepon = FALSE;
+        error = SELECT_ERROR;
+        failf(data, "Connection aborted");
+      }
+      else {
+        /* we got a whole chunk of data, which can be anything from one
+         * byte to a set of lines and possibly just a piece of the last
+         * line */
+        int i;
+
+        nread += gotbytes;
+        for(i = 0; i < gotbytes; ptr++, i++) {
+          perline++; /* amount of bytes in this line so far */
+          if(*ptr=='\n') {
+            /* a newline is CRLF in ftp-talk, so the CR is ignored as
+               the line isn't really terminated until the LF comes */
+
+            /* output debug output if that is requested */
+            if(data->set.verbose) {
+              fputs("< ", data->set.err);
+              fwrite(line_start, perline, 1, data->set.err);
+              /* no need to output LF here, it is part of the data */
+            }
+            
+            if('\r' == line_start[0]) {
+              /* end of headers */
+              keepon=FALSE;
+              break; /* breaks out of loop, not switch */
+            }
+
+            if(2 == sscanf(line_start, "HTTP/1.%d %d",
+                           &subversion,
+                           &httperror)) {
+              ;
+            }
+
+            perline=0; /* line starts over here */
+            line_start = ptr+1;
+          }
+        }
+      }
+      break;
+    } /* switch */
+  } /* while there's buffer left and loop is requested */
+
+  if(error)
+    return CURLE_READ_ERROR;
+
   if(200 != httperror) {
     if(407 == httperror)
       /* Added Nov 6 1998 */
@@ -307,15 +398,18 @@ CURLcode Curl_http_connect(struct connectdata *conn)
    * us to the host we want to talk to.  Only after the connect
    * has occured, can we start talking SSL
    */
-  if (conn->protocol & PROT_HTTPS) {
-    if (data->change.proxy) {
-      /* HTTPS through a proxy can only be done with a tunnel */
-      result = Curl_ConnectHTTPProxyTunnel(conn, conn->firstsocket,
-                                           conn->hostname, conn->remote_port);
-      if(CURLE_OK != result)
-        return result;
-    }
 
+  if(data->change.proxy &&
+     ((conn->protocol & PROT_HTTPS) || data->set.tunnel_thru_httpproxy)) {
+
+    /* either HTTPS over proxy, OR explicitly asked for a tunnel */
+    result = Curl_ConnectHTTPProxyTunnel(conn, conn->firstsocket,
+                                         conn->hostname, conn->remote_port);
+    if(CURLE_OK != result)
+      return result;
+  }    
+
+  if(conn->protocol & PROT_HTTPS) {
     /* now, perform the SSL initialization for this socket */
     result = Curl_SSLConnect(conn);
     if(result)
@@ -352,6 +446,13 @@ CURLcode Curl_http_done(struct connectdata *conn)
     *bytecount = http->readbytecount + http->writebytecount;
   }
 
+  if(0 == (http->readbytecount + conn->headerbytecount)) {
+    /* nothing was read from the HTTP server, this can't be right
+       so we return an error here */
+    failf(data, "Empty reply from server\n");
+    return CURLE_GOT_NOTHING;
+  }
+
   return CURLE_OK;
 }
 
@@ -432,7 +533,9 @@ CURLcode Curl_http(struct connectdata *conn)
                              host, ppath,
                              conn->protocol&PROT_HTTPS?TRUE:FALSE);
   }
-  if ((data->change.proxy) && !(conn->protocol&PROT_HTTPS))  {
+  if (data->change.proxy &&
+      !data->set.tunnel_thru_httpproxy &&
+      !(conn->protocol&PROT_HTTPS))  {
     /* The path sent to the proxy is in fact the entire URL */
     ppath = data->change.url;
   }
@@ -555,6 +658,10 @@ CURLcode Curl_http(struct connectdata *conn)
   }
 
   do {
+    /* Use 1.1 unless the use specificly asked for 1.0 */
+    const char *httpstring=
+      data->set.httpversion==CURL_HTTP_VERSION_1_0?"1.0":"1.1";
+
     send_buffer *req_buffer;
     struct curl_slist *headers=data->set.headers;
 
@@ -564,7 +671,7 @@ CURLcode Curl_http(struct connectdata *conn)
     /* add the main request stuff */
     add_bufferf(req_buffer,
                 "%s " /* GET/HEAD/POST/PUT */
-                "%s HTTP/1.1\r\n" /* path */
+                "%s HTTP/%s\r\n" /* path */
                 "%s" /* proxyuserpwd */
                 "%s" /* userpwd */
                 "%s" /* range */
@@ -580,7 +687,7 @@ CURLcode Curl_http(struct connectdata *conn)
                  ((HTTPREQ_POST == data->set.httpreq) ||
                   (HTTPREQ_POST_FORM == data->set.httpreq))?"POST":
                  (HTTPREQ_PUT == data->set.httpreq)?"PUT":"GET"),
-                ppath,
+                ppath, httpstring,
                 (conn->bits.proxy_user_passwd &&
                  conn->allocptr.proxyuserpwd)?conn->allocptr.proxyuserpwd:"",
                 (conn->bits.user_passwd && conn->allocptr.userpwd)?
@@ -644,7 +751,7 @@ CURLcode Curl_http(struct connectdata *conn)
 
 #ifdef HAVE_STRFTIME
       /* format: "Tue, 15 Nov 1994 12:45:26 GMT" */
-      strftime(buf, BUFSIZE-1, "%a, %d %b %Y %H:%M:%S %Z", thistime);
+      strftime(buf, BUFSIZE-1, "%a, %d %b %Y %H:%M:%S GMT", thistime);
 #else
       /* TODO: Right, we *could* write a replacement here */
       strcpy(buf, "no strftime() support");
@@ -729,14 +836,16 @@ CURLcode Curl_http(struct connectdata *conn)
       Curl_pgrsSetUploadSize(data, http->postsize);
 
       /* fire away the whole request to the server */
-      data->info.request_size = 
-        add_buffer_send(conn->firstsocket, conn, req_buffer);
-
-      /* setup variables for the upcoming transfer */
-      result = Curl_Transfer(conn, conn->firstsocket, -1, TRUE,
-                             &http->readbytecount,
-                             conn->firstsocket,
-                             &http->writebytecount);
+      result = add_buffer_send(conn->firstsocket, conn, req_buffer,
+                               &data->info.request_size);
+      if(result)
+        failf(data, "Failed sending POST request");
+      else
+        /* setup variables for the upcoming transfer */
+        result = Curl_Transfer(conn, conn->firstsocket, -1, TRUE,
+                               &http->readbytecount,
+                               conn->firstsocket,
+                               &http->writebytecount);
       if(result) {
         Curl_formclean(http->sendit); /* free that whole lot */
         return result;
@@ -757,14 +866,16 @@ CURLcode Curl_http(struct connectdata *conn)
       Curl_pgrsSetUploadSize(data, data->set.infilesize);
 
       /* this sends the buffer and frees all the buffer resources */
-      data->info.request_size = 
-        add_buffer_send(conn->firstsocket, conn, req_buffer);
-
-      /* prepare for transfer */
-      result = Curl_Transfer(conn, conn->firstsocket, -1, TRUE,
-                        &http->readbytecount,
-                        conn->firstsocket,
-                        &http->writebytecount);
+      result = add_buffer_send(conn->firstsocket, conn, req_buffer,
+                               &data->info.request_size);
+      if(result)
+        failf(data, "Faied sending POST request");
+      else
+        /* prepare for transfer */
+        result = Curl_Transfer(conn, conn->firstsocket, -1, TRUE,
+                               &http->readbytecount,
+                               conn->firstsocket,
+                               &http->writebytecount);
       if(result)
         return result;
       
@@ -814,12 +925,15 @@ CURLcode Curl_http(struct connectdata *conn)
         add_buffer(req_buffer, "\r\n", 2);
 
       /* issue the request */
-      data->info.request_size = 
-        add_buffer_send(conn->firstsocket, conn, req_buffer);
+      result = add_buffer_send(conn->firstsocket, conn, req_buffer,
+                               &data->info.request_size);
 
-      /* HTTP GET/HEAD download: */
-      result = Curl_Transfer(conn, conn->firstsocket, -1, TRUE, bytecount,
-                        -1, NULL); /* nothing to upload */
+      if(result)
+        failf(data, "Failed sending HTTP request");
+      else
+        /* HTTP GET/HEAD download: */
+        result = Curl_Transfer(conn, conn->firstsocket, -1, TRUE, bytecount,
+                               -1, NULL); /* nothing to upload */
     }
     if(result)
       return result;
@@ -834,6 +948,6 @@ CURLcode Curl_http(struct connectdata *conn)
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
