@@ -1,6 +1,6 @@
 /****************************************************************************
  *
- * $Id: mprintf.c,v 1.12 2001/09/07 04:01:32 bumblebury Exp $
+ * $Id: mprintf.c,v 1.16 2001/11/27 07:27:32 bagder Exp $
  *
  *************************************************************************
  *
@@ -92,14 +92,7 @@
  *
  ****************************************************************************/
 
-static const char rcsid[] = "@(#)$Id: mprintf.c,v 1.12 2001/09/07 04:01:32 bumblebury Exp $";
-
-/*
- * To test:
- *
- * Use WIDTH, PRECISION and NUMBERED ARGUMENT combined.
- */
-
+#include "setup.h"
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -909,14 +902,14 @@ static int dprintf_formatf(
 	}
 	else {
 	  /* Write "(nil)" for a nil pointer.  */
-	  static char nil[] = "(nil)";
+	  static char strnil[] = "(nil)";
 	  register char *point;
 	  
-	  width -= sizeof(nil) - 1;
+	  width -= sizeof(strnil) - 1;
 	  if (p->flags & FLAGS_LEFT)
 	    while (width-- > 0)
 	      OUTCHAR(' ');
-	  for (point = nil; *point != '\0'; ++point)
+	  for (point = strnil; *point != '\0'; ++point)
 	    OUTCHAR(*point);
 	  if (! (p->flags & FLAGS_LEFT))
 	    while (width-- > 0)
@@ -1237,6 +1230,6 @@ int main()
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
