@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sendf.c,v 1.29 2001/09/07 04:01:32 bumblebury Exp $
+ * $Id: sendf.c,v 1.33 2001/12/04 13:03:09 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -27,6 +27,10 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>	/* required for send() & recv() prototypes */
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -140,17 +144,20 @@ void Curl_failf(struct SessionHandle *data, const char *fmt, ...)
 {
   va_list ap;
   va_start(ap, fmt);
-  if(data->set.errorbuffer)
+  if(data->set.errorbuffer && !data->state.errorbuf) {
     vsnprintf(data->set.errorbuffer, CURL_ERROR_SIZE, fmt, ap);
+    data->state.errorbuf = TRUE; /* wrote error string */
+  }
   va_end(ap);
 }
 
 /* Curl_sendf() sends formated data to the server */
-size_t Curl_sendf(int sockfd, struct connectdata *conn,
-                  const char *fmt, ...)
+CURLcode Curl_sendf(int sockfd, struct connectdata *conn,
+                    const char *fmt, ...)
 {
   struct SessionHandle *data = conn->data;
   size_t bytes_written;
+  CURLcode result;
   char *s;
   va_list ap;
   va_start(ap, fmt);
@@ -162,11 +169,11 @@ size_t Curl_sendf(int sockfd, struct connectdata *conn,
     fprintf(data->set.err, "> %s", s);
 
   /* Write the buffer to the socket */
-  Curl_write(conn, sockfd, s, strlen(s), &bytes_written);
+  result = Curl_write(conn, sockfd, s, strlen(s), &bytes_written);
 
   free(s); /* free the output string */
 
-  return bytes_written;
+  return result;
 }
 
 /*
@@ -211,7 +218,7 @@ CURLcode Curl_write(struct connectdata *conn, int sockfd,
 #endif
 
   *written = bytes_written;
-  return CURLE_OK;
+  return (bytes_written==len)?CURLE_OK:CURLE_WRITE_ERROR;
 }
 
 /* client_write() sends data to the write callback(s)
@@ -296,6 +303,6 @@ CURLcode Curl_read(struct connectdata *conn, int sockfd,
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
