@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: ssluse.c,v 1.38 2001/09/19 21:49:11 bagder Exp $
+ * $Id: ssluse.c,v 1.41 2001/11/14 07:11:39 bagder Exp $
  *****************************************************************************/
 
 /*
@@ -292,6 +292,8 @@ void Curl_SSL_cleanup(void)
     /* EVP_cleanup() removes all ciphers and digests from the
        table. */
     EVP_cleanup();
+
+    init_ssl=0; /* not inited any more */
   }
 #else
   /* SSL disabled, do nothing */
@@ -540,15 +542,21 @@ Curl_SSLConnect(struct connectdata *conn)
 
   /* Make funny stuff to get random input */
   random_the_seed(conn);
-    
+
+  /* check to see if we've been told to use an explicit SSL/TLS version */
   switch(data->set.ssl.version) {
   default:
+  case CURL_SSLVERSION_DEFAULT:
+    /* we try to figure out version */
     req_method = SSLv23_client_method();
     break;
-  case 2:
+  case CURL_SSLVERSION_TLSv1:
+    req_method = TLSv1_client_method();
+    break;
+  case CURL_SSLVERSION_SSLv2:
     req_method = SSLv2_client_method();
     break;
-  case 3:
+  case CURL_SSLVERSION_SSLv3:
     req_method = SSLv3_client_method();
     break;
   }
@@ -725,6 +733,6 @@ Curl_SSLConnect(struct connectdata *conn)
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
