@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 2001, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * In order to be useful for every potential user, curl and libcurl are
  * dual-licensed under the MPL and the MIT/X-derivate licenses.
@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: transfer.c,v 1.55 2001/09/12 12:02:12 bagder Exp $
+ * $Id: transfer.c,v 1.71 2001/12/04 09:14:41 bagder Exp $
  *****************************************************************************/
 
 #include "setup.h"
@@ -220,8 +220,6 @@ Transfer(struct connectdata *c_conn)
 
   hbufp = data->state.headerbuff;
 
-  myalarm (0);			/* switch off the alarm-style timeout */
-
   now = Curl_tvnow();
   start = now;
 
@@ -307,6 +305,8 @@ Transfer(struct connectdata *c_conn)
         }
 	break;
       default:
+        if ((bytecount == 0) && (writebytecount == 0))
+          Curl_pgrsTime(data, TIMER_STARTTRANSFER);
         if((keepon & KEEP_READ) && FD_ISSET(conn->sockfd, &readfd)) {
           /* read! */
           urg = Curl_read(conn, conn->sockfd, buf, BUFSIZE -1, &nread);
@@ -452,12 +452,14 @@ Transfer(struct connectdata *c_conn)
                 if (data->set.http_include_header)
                   writetype |= CLIENTWRITE_BODY;
 
-                urg = Curl_client_write(data, writetype, data->state.headerbuff,
+                urg = Curl_client_write(data, writetype,
+                                        data->state.headerbuff,
                                         p - data->state.headerbuff);
                 if(urg)
                   return urg;
 
                 data->info.header_size += p - data->state.headerbuff;
+                conn->headerbytecount += p - data->state.headerbuff;
 
                 if(!header) {
                   /*
@@ -520,13 +522,8 @@ Transfer(struct connectdata *c_conn)
                   data->info.httpversion = httpversion;
 
                   /* 404 -> URL not found! */
-                  if (
-                      ( ((data->set.http_follow_location) &&
-                         (httpcode >= 400))
-                        ||
-                        (!data->set.http_follow_location &&
-                         (httpcode >= 300)))
-                      && (data->set.http_fail_on_error)) {
+                  if (data->set.http_fail_on_error &&
+                      (httpcode >= 400)) {
                     /* If we have been told to fail hard on HTTP-errors,
                        here is the check for that: */
                     /* serious error, go home! */
@@ -620,14 +617,14 @@ Transfer(struct connectdata *c_conn)
               }
               else if(data->cookies &&
                       strnequal("Set-Cookie:", p, 11)) {
-                Curl_cookie_add(data->cookies, TRUE, &p[12]);
+                Curl_cookie_add(data->cookies, TRUE, &p[12], conn->name);
               }
               else if(strnequal("Last-Modified:", p,
                                 strlen("Last-Modified:")) &&
                       (data->set.timecondition || data->set.get_filetime) ) {
                 time_t secs=time(NULL);
                 timeofdoc = curl_getdate(p+strlen("Last-Modified:"), &secs);
-                if(data->set.get_filetime)
+                if(data->set.get_filetime>=0)
                   data->info.filetime = timeofdoc;
               }
               else if ((httpcode >= 300 && httpcode < 400) &&
@@ -844,6 +841,12 @@ Transfer(struct connectdata *c_conn)
         break;
       }
 
+      /* Update read/write counters */
+      if(conn->bytecountp)
+        *conn->bytecountp = bytecount; /* read count */
+      if(conn->writebytecountp)
+        *conn->writebytecountp = writebytecount; /* write count */
+
       now = Curl_tvnow();
       if(Curl_pgrsUpdate(conn))
         urg = CURLE_ABORTED_BY_CALLBACK;
@@ -861,11 +864,13 @@ Transfer(struct connectdata *c_conn)
         conn->upload_bufsize=(long)min(data->progress.ulspeed, BUFSIZE);
       }
 
-      if (data->set.timeout && (Curl_tvdiff (now, start) > data->set.timeout)) {
+      if (data->set.timeout &&
+          ((Curl_tvdiff(now, start)/1000) >= data->set.timeout)) {
 	failf (data, "Operation timed out with %d out of %d bytes received",
 	       bytecount, conn->size);
 	return CURLE_OPERATION_TIMEOUTED;
       }
+
     }
   }
 
@@ -888,11 +893,6 @@ Transfer(struct connectdata *c_conn)
   if(Curl_pgrsUpdate(conn))
     return CURLE_ABORTED_BY_CALLBACK;
 
-  if(conn->bytecountp)
-    *conn->bytecountp = bytecount; /* read count */
-  if(conn->writebytecountp)
-    *conn->writebytecountp = writebytecount; /* write count */
-
   return CURLE_OK;
 }
 
@@ -902,6 +902,10 @@ CURLcode Curl_perform(struct SessionHandle *data)
   struct connectdata *conn=NULL;
   bool port=TRUE; /* allow data->set.use_port to set port to use */
   char *newurl = NULL; /* possibly a new URL to follow to! */
+#ifdef HAVE_SIGNAL
+  /* storage for the previous bag^H^H^HSIGPIPE signal handler :-) */
+  void (*prev_signal)(int sig);
+#endif
 
   if(!data->change.url)
     /* we can't do anything wihout URL */
@@ -916,17 +920,55 @@ CURLcode Curl_perform(struct SessionHandle *data)
 
   data->set.followlocation=0; /* reset the location-follow counter */
   data->state.this_is_a_follow = FALSE; /* reset this */
+  data->state.errorbuf = FALSE; /* no error has occurred */
+
+#if defined(HAVE_SIGNAL) && defined(SIGPIPE)
+  /*************************************************************
+   * Tell signal handler to ignore SIGPIPE
+   *************************************************************/
+  prev_signal = signal(SIGPIPE, SIG_IGN);
+#endif  
 
   Curl_initinfo(data); /* reset session-specific information "variables" */
-
   Curl_pgrsStartNow(data);
 
+  /*
+   * It is important that there is NO 'return' from this function any any
+   * other place than falling down the bottom! This is because we have cleanup
+   * stuff that must be done before we get back, and that is only performed
+   * after this do-while loop.
+   */
+
   do {
     Curl_pgrsTime(data, TIMER_STARTSINGLE);
     res = Curl_connect(data, &conn, port);
     if(res == CURLE_OK) {
       res = Curl_do(conn);
+
+      if((CURLE_WRITE_ERROR == res) && conn->bits.reuse) {
+        /* This was a re-use of a connection and we got a write error in the
+         * DO-phase. Then we DISCONNECT this connection and have another
+         * attempt to CONNECT and then DO again! The retry cannot possibly
+         * find another connection to re-use, since we only keep one possible
+         * connection for each.
+         */
+
+        infof(data, "The re-used connection seems dead, get a new one\n");
+
+        conn->bits.close = TRUE; /* enforce close of this connetion */
+        res = Curl_done(conn);   /* we are so done with this */
+        if(CURLE_OK == res) {
+          /* Now, redo the connect */
+          res = Curl_connect(data, &conn, port);
+          if(CURLE_OK == res)
+            /* ... finally back to actually retry the DO phase */
+            res = Curl_do(conn);
+        }
+      }
+
       if(res == CURLE_OK) {
+        CURLcode res2; /* just a local extra result container */
+
         if(conn->protocol&PROT_FTPS)
           /* FTPS, disable ssl while transfering data */
           conn->ssl.use = FALSE;
@@ -935,15 +977,24 @@ CURLcode Curl_perform(struct SessionHandle *data)
           /* FTPS, enable ssl again after havving transferred data */
           conn->ssl.use = TRUE;
 
-        if(res == CURLE_OK) {
+        if(res == CURLE_OK)
           /*
            * We must duplicate the new URL here as the connection data
            * may be free()ed in the Curl_done() function.
            */
           newurl = conn->newurl?strdup(conn->newurl):NULL;
+        else
+          /* The transfer phase returned error, we mark the connection to get
+           * closed to prevent being re-used. This is becasue we can't
+           * possibly know if the connection is in a good shape or not now. */
+          conn->bits.close = TRUE;
 
-          res = Curl_done(conn);
-        }
+        /* Always run Curl_done(), even if some of the previous calls
+           failed, but return the previous (original) error code */
+        res2 = Curl_done(conn);
+
+        if(CURLE_OK == res)
+          res = res2;
       }
 
       /*
@@ -986,7 +1037,7 @@ CURLcode Curl_perform(struct SessionHandle *data)
           data->change.referer_alloc = TRUE; /* yes, free this later */
         }
 
-        if(2 != sscanf(newurl, "%15[^:]://%c", prot, &letter)) {
+        if(2 != sscanf(newurl, "%15[^?&/:]://%c", prot, &letter)) {
           /***
            *DANG* this is an RFC 2068 violation. The URL is supposed
            to be absolute and this doesn't seem to be that!
@@ -1003,8 +1054,10 @@ CURLcode Curl_perform(struct SessionHandle *data)
              point to read-only data */
           char *url_clone=strdup(data->change.url);
 
-          if(!url_clone)
-            return CURLE_OUT_OF_MEMORY;
+          if(!url_clone) {
+            res = CURLE_OUT_OF_MEMORY;
+            break; /* skip out of this loop NOW */
+          }
 
           /* protsep points to the start of the host name */
           protsep=strstr(url_clone, "//");
@@ -1038,8 +1091,10 @@ CURLcode Curl_perform(struct SessionHandle *data)
                                  1 + /* possible slash */
                                  strlen(newurl) + 1/* zero byte */);
 
-          if(!newest)
-            return CURLE_OUT_OF_MEMORY;
+          if(!newest) {
+            res = CURLE_OUT_OF_MEMORY;
+            break; /* go go go out from this loop */
+          }
           sprintf(newest, "%s%s%s", url_clone, ('/' == newurl[0])?"":"/",
                   newurl);
           free(newurl);
@@ -1127,9 +1182,10 @@ CURLcode Curl_perform(struct SessionHandle *data)
   if(newurl)
     free(newurl);
 
-  /* make sure the alarm is switched off! */
-  if(data->set.timeout || data->set.connecttimeout)
-    myalarm(0);
+#if defined(HAVE_SIGNAL) && defined(SIGPIPE)
+  /* restore the signal handler for SIGPIPE before we get back */
+  signal(SIGPIPE, prev_signal);
+#endif  
 
   return res;
 }
@@ -1166,6 +1222,6 @@ Curl_Transfer(struct connectdata *c_conn, /* connection data */
  * local variables:
  * eval: (load-file "../curl-mode.el")
  * end:
- * vim600: et sw=2 ts=2 sts=2 tw=78 fdm=marker
- * vim<600: et sw=2 ts=2 sts=2 tw=78
+ * vim600: fdm=marker
+ * vim: et sw=2 ts=2 sts=2 tw=78
  */
