@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: main.c,v 1.95 2001/09/11 22:36:03 bagder Exp $
+ * $Id: main.c,v 1.104 2001/11/30 09:26:06 bagder Exp $
  *****************************************************************************/
 
 /* This is now designed to have its own local setup.h */
@@ -61,11 +61,11 @@
 #endif
 
 #ifdef HAVE_UTIME_H
- #include <utime.h>
+#include <utime.h>
 #else
- #ifdef HAVE_SYS_UTIME_H
- #include <sys/utime.h>
- #endif
+#ifdef HAVE_SYS_UTIME_H
+#include <sys/utime.h>
+#endif
 #endif
 
 /* The last #include file should be: */
@@ -313,6 +313,7 @@ static void help(void)
        " -d/--data <data>   HTTP POST data (H)\n"
        "    --data-ascii <data>   HTTP POST ASCII data (H)\n"
        "    --data-binary <data>  HTTP POST binary data (H)\n"
+       "    --disable-epsv  Prevents curl from using EPSV (F)\n"
        " -D/--dump-header <file> Write the headers to this file\n"
        "    --egd-file <file> EGD socket path for random data (SSL)\n"
        " -e/--referer       Referer page (H)");
@@ -363,6 +364,8 @@ static void help(void)
        " -Y/--speed-limit   Stop transfer if below speed-limit for 'speed-time' secs\n"
        " -z/--time-cond <time> Includes a time condition to the server (H)\n"
        " -Z/--max-redirs <num> Set maximum number of redirections allowed (H)\n"
+       " -0/--http1.0       Force usage of HTTP 1.0 (H)\n"
+       " -1/--tlsv1         Force usage of TLSv1 (H)\n"
        " -2/--sslv2         Force usage of SSLv2 (H)\n"
        " -3/--sslv3         Force usage of SSLv3 (H)");
   puts(" -#/--progress-bar  Display transfer progress as a progress bar\n"
@@ -385,6 +388,7 @@ struct Configurable {
   char *cookiefile; /* read from this file */
   bool use_resume;
   bool resume_from_current;
+  bool disable_epsv;
   int resume_from;
   char *postfields;
   long postfieldsize;
@@ -421,6 +425,7 @@ struct Configurable {
   bool crlf;
   char *customrequest;
   char *krb4level;
+  long httpversion;
   bool progressmode;
   bool nobuffer;
   bool globoff;
@@ -434,7 +439,7 @@ struct Configurable {
   struct curl_slist *postquote;
 
   long ssl_version;
-  TimeCond timecond;
+  curl_TimeCond timecond;
   time_t condtime;
 
   struct curl_slist *headers;
@@ -560,7 +565,259 @@ struct getout *new_getout(struct Configurable *config)
   return node;
 }
 
+/* Structure for storing the information needed to build a multiple files
+ * section
+*/
+struct multi_files {
+  struct curl_forms   form;
+  struct multi_files *next;
+};
 
+/* Add a new list entry possibly with a type_name
+ */
+static void *AddMultiFiles (const char *file_name,
+                            const char *type_name,
+                            struct multi_files **multi_start,
+                            struct multi_files **multi_current)
+{
+  struct multi_files *multi;
+  struct multi_files *multi_type = NULL;
+  multi = (struct multi_files *)malloc(sizeof(struct multi_files));
+  if (multi) {
+    memset(multi, 0, sizeof(struct multi_files));
+    multi->form.option = CURLFORM_FILE;
+    multi->form.value = file_name;
+  }
+  else
+    return NULL;
+  if (type_name) {
+    multi_type = (struct multi_files *)malloc(sizeof(struct multi_files));
+    if (multi_type) {
+      memset(multi_type, 0, sizeof(struct multi_files));
+      multi_type->form.option = CURLFORM_CONTENTTYPE;
+      multi_type->form.value = type_name;
+      multi->next = multi_type;
+    }
+    else {
+      free (multi);
+      return NULL;
+    }
+  }
+  if (!*multi_start)
+    *multi_start = multi;
+  if (!*multi_current) {
+    if (multi_type)
+      *multi_current = multi_type;
+    else
+      *multi_current = multi;
+  }
+  else {
+    if (multi_type) {
+      (*multi_current)->next = multi;
+      *multi_current = multi_type;
+    }
+    else {
+      (*multi_current)->next = multi;
+      *multi_current = multi;
+    }
+  }
+  return *multi_current;
+}
+
+/* Free the items of the list.
+ */
+static void FreeMultiInfo (struct multi_files **multi_start)
+{
+  struct multi_files *multi;
+  while (*multi_start) {
+    multi = *multi_start;
+    *multi_start = (*multi_start)->next;
+    free (multi);
+  }
+}
+
+/***************************************************************************
+ *
+ * formparse()
+ *	
+ * Reads a 'name=value' paramter and builds the appropriate linked list.
+ *
+ * Specify files to upload with 'name=@filename'. Supports specified
+ * given Content-Type of the files. Such as ';type=<content-type>'.
+ *
+ * You may specify more than one file for a single name (field). Specify
+ * multiple files by writing it like:
+ *
+ * 'name=@filename,filename2,filename3'
+ *
+ * If you want content-types specified for each too, write them like:
+ *
+ * 'name=@filename;type=image/gif,filename2,filename3'
+ *
+ * Does use curl_formadd to fulfill it's job. Is heavily based on the
+ * old curl_formparse code.
+ *
+ ***************************************************************************/
+
+#define FORM_FILE_SEPARATOR ','
+#define FORM_TYPE_SEPARATOR ';'
+
+static int formparse(char *input,
+                     struct HttpPost **httppost,
+                     struct HttpPost **last_post)
+{
+  /* nextarg MUST be a string in the format 'name=contents' and we'll
+     build a linked list with the info */
+  char name[256];
+  char *contents;
+  char major[128];
+  char minor[128];
+  char *contp;
+  const char *type = NULL;
+  char *sep;
+  char *sep2;
+
+  /* Preallocate contents to the length of input to make sure we don't
+     overwrite anything. */
+  contents = malloc(strlen(input));
+  contents[0] = '\000';
+ 
+  if(1 <= sscanf(input, "%255[^=]=%[^\n]", name, contents)) {
+    /* the input was using the correct format */
+    contp = contents;
+
+    if('@' == contp[0]) {
+      struct multi_files *multi_start = NULL, *multi_current = NULL;
+      /* we use the @-letter to indicate file name(s) */
+      contp++;
+
+      multi_start = multi_current=NULL;
+
+      do {
+	/* since this was a file, it may have a content-type specifier
+	   at the end too */
+
+	sep=strchr(contp, FORM_TYPE_SEPARATOR);
+	sep2=strchr(contp, FORM_FILE_SEPARATOR);
+
+	/* pick the closest */
+	if(sep2 && (sep2 < sep)) {
+	  sep = sep2;
+
+	  /* no type was specified! */
+	}
+	if(sep) {
+
+	  /* if we got here on a comma, don't do much */
+	  if(FORM_FILE_SEPARATOR != *sep)
+	    type = strstr(sep+1, "type=");
+	  else
+	    type=NULL;
+
+	  *sep=0; /* terminate file name at separator */
+
+	  if(type) {
+	    type += strlen("type=");
+	    
+	    if(2 != sscanf(type, "%127[^/]/%127[^,\n]",
+			   major, minor)) {
+	      fprintf(stderr, "Illegally formatted content-type field!\n");
+              free(contents);
+              FreeMultiInfo (&multi_start);
+	      return 2; /* illegal content-type syntax! */
+	    }
+	    /* now point beyond the content-type specifier */
+	    sep = (char *)type + strlen(major)+strlen(minor)+1;
+
+	    /* find the following comma */
+	    sep=strchr(sep, FORM_FILE_SEPARATOR);
+	  }
+	}
+	else {
+	  type=NULL;
+	  sep=strchr(contp, FORM_FILE_SEPARATOR);
+	}
+	if(sep) {
+	  /* the next file name starts here */
+	  *sep =0;
+	  sep++;
+	}
+        /* if type == NULL curl_formadd takes care of the problem */
+
+        if (!AddMultiFiles (contp, type, &multi_start, &multi_current)) {
+          fprintf(stderr, "Error building form post!\n");
+          free(contents);
+          FreeMultiInfo (&multi_start);
+          return 3;
+        }
+	contp = sep; /* move the contents pointer to after the separator */
+      } while(sep && *sep); /* loop if there's another file name */
+      /* now we add the multiple files section */
+      if (multi_start) {
+        struct curl_forms *forms = NULL;
+        struct multi_files *ptr = multi_start;
+        unsigned int i, count = 0;
+        while (ptr) {
+          ptr = ptr->next;
+          ++count;
+        }
+        forms =
+          (struct curl_forms *)malloc((count+1)*sizeof(struct curl_forms));
+        if (!forms)
+        {
+          fprintf(stderr, "Error building form post!\n");
+          free(contents);
+          FreeMultiInfo (&multi_start);
+          return 4;
+        }
+        for (i = 0, ptr = multi_start; i < count; ++i, ptr = ptr->next)
+        {
+          forms[i].option = ptr->form.option;
+          forms[i].value = ptr->form.value;
+        }
+        forms[count].option = CURLFORM_END;
+        FreeMultiInfo (&multi_start);
+        if (curl_formadd (httppost, last_post,
+                          CURLFORM_COPYNAME, name,
+                          CURLFORM_ARRAY, forms, CURLFORM_END) != 0) {
+          fprintf(stderr, "curl_formadd failed!\n");
+          free(forms);
+          free(contents);
+          return 5;
+        }
+        free(forms);
+      }
+    }
+    else {
+      if( contp[0]=='<' ) {
+        if (curl_formadd (httppost, last_post,
+                          CURLFORM_COPYNAME, name,
+                          CURLFORM_FILECONTENT, contp+1, CURLFORM_END) != 0) {
+          fprintf(stderr, "curl_formadd failed!\n");
+          free(contents);
+          return 6;
+        }
+      }
+      else {
+        if (curl_formadd (httppost, last_post,
+                          CURLFORM_COPYNAME, name,
+                          CURLFORM_COPYCONTENTS, contp, CURLFORM_END) != 0) {
+          fprintf(stderr, "curl_formadd failed!\n");
+          free(contents);
+          return 7;
+        }
+      }
+    }
+
+  }
+  else {
+    fprintf(stderr, "Illegally formatted input field!\n");
+    free(contents);
+    return 1;
+  }
+  free(contents);
+  return 0;
+}
 
 
 typedef enum {
@@ -607,7 +864,10 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     {"5b", "egd-file",   TRUE},
     {"5c", "connect-timeout", TRUE},
     {"5d", "ciphers",    TRUE},
+    {"5e", "disable-epsv", FALSE},
 
+    {"0", "http1.0",     FALSE},
+    {"1", "tlsv1",       FALSE},
     {"2", "sslv2",       FALSE},
     {"3", "sslv3",       FALSE},
     {"a", "append",      FALSE},
@@ -771,6 +1031,9 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
       case 'd': /* ciphers */
         GetStr(&config->cipher_list, nextarg);
         break;
+      case 'e': /* --disable-epsv */
+        config->disable_epsv ^= TRUE;
+        break;
       default: /* the URL! */
         {
           struct getout *url;
@@ -801,13 +1064,21 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     case '#': /* added 19990617 larsa */
       config->progressmode ^= CURL_PROGRESS_BAR;
       break;
+    case '0': 
+      /* HTTP version 1.0 */
+      config->httpversion = CURL_HTTP_VERSION_1_0;
+      break;
+    case '1':
+      /* TLS version 1 */
+      config->ssl_version = CURL_SSLVERSION_TLSv1;
+      break;
     case '2': 
       /* SSL version 2 */
-      config->ssl_version = 2;
+      config->ssl_version = CURL_SSLVERSION_SSLv2;
       break;
     case '3': 
-      /* SSL version 2 */
-      config->ssl_version = 3;
+      /* SSL version 3 */
+      config->ssl_version = CURL_SSLVERSION_SSLv3;
       break;
     case 'a':
       /* This makes the FTP sessions use APPE instead of STOR */
@@ -947,9 +1218,9 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     case 'F':
       /* "form data" simulation, this is a little advanced so lets do our best
 	 to sort this out slowly and carefully */
-      if(curl_formparse(nextarg,
-                        &config->httppost,
-                        &config->last_post))
+      if(formparse(nextarg,
+                   &config->httppost,
+                   &config->last_post))
 	return PARAM_BAD_USE;
       if(SetHTTPrequest(HTTPREQ_POST, &config->httpreq))
         return PARAM_BAD_USE;
@@ -1193,7 +1464,7 @@ static ParameterError getparameter(char *flag, /* f or -long-flag */
     }
     hit = -1;
 
-  } while(!singleopt && *++parse && !*usedarg);
+  } while(!longopt && !singleopt && *++parse && !*usedarg);
 
   return PARAM_OK;
 }
@@ -1532,6 +1803,8 @@ void free_config_fields(struct Configurable *config)
     curl_formfree(config->httppost);
   if(config->cacert)
     free(config->cacert);
+  if(config->cookiejar)
+    free(config->cookiejar);
 
   curl_slist_free_all(config->quote); /* checks for config->quote == NULL */
   curl_slist_free_all(config->postquote); /*  */
@@ -1569,9 +1842,11 @@ operate(struct Configurable *config, int argc, char *argv[])
   char *httpgetfields=NULL;
 
   CURL *curl;
-  int res;
+  int res = 0;
   int i;
 
+  errorbuffer[0]=0; /* prevent junk from being output */
+
 #ifdef MALLOCDEBUG
   /* this sends all memory debug messages to a logfile named memdump */
   curl_memdebug("memdump");
@@ -2031,9 +2306,16 @@ operate(struct Configurable *config, int argc, char *argv[])
       curl_easy_setopt(curl, CURLOPT_EGDSOCKET, config->egd_file);
       curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, config->connecttimeout);
 
-      /* debug */
       if(config->cipher_list)
         curl_easy_setopt(curl, CURLOPT_SSL_CIPHER_LIST, config->cipher_list);
+
+      if(config->httpversion)
+        curl_easy_setopt(curl, CURLOPT_HTTP_VERSION, config->httpversion);
+
+      /* new in libcurl 7.9.2: */
+      if(config->disable_epsv)
+        /* disable it */
+        curl_easy_setopt(curl, CURLOPT_FTP_USE_EPSV, FALSE);
       
       res = curl_easy_perform(curl);
         
@@ -2053,8 +2335,6 @@ operate(struct Configurable *config, int argc, char *argv[])
       if(config->headerfile && !headerfilep && heads.stream)
         fclose(heads.stream);
 
-      if(urlbuffer)
-        free(urlbuffer);
       if (outfile && !strequal(outfile, "-") && outs.stream)
         fclose(outs.stream);
 
